<?php
/**
 * MakePay Invision Community application bootstrap.
 */

namespace IPS\makepay;

if (!\defined('\IPS\SUITE_UNIQUE_KEY')) {
    exit;
}

class _Application extends \IPS\Application
{
    public function get__icon(): string
    {
        return 'credit-card';
    }
}
